# -*- coding: UTF-8 -*

"""Model : site config  WORK IN PROGRESS #WIP
"""

from app import db, log
from app.scodoc import bonus_sport
from app.scodoc.sco_exceptions import ScoValueError
import functools

from app.scodoc.sco_codes_parcours import (
    ADC,
    ADJ,
    ADM,
    AJ,
    ATB,
    ATJ,
    ATT,
    CMP,
    DEF,
    DEM,
    NAR,
    RAT,
)

CODES_SCODOC_TO_APO = {
    ADC: "ADMC",
    ADJ: "ADM",
    ADM: "ADM",
    AJ: "AJ",
    ATB: "AJAC",
    ATJ: "AJAC",
    ATT: "AJAC",
    CMP: "COMP",
    DEF: "NAR",
    DEM: "NAR",
    NAR: "NAR",
    RAT: "ATT",
}


def code_scodoc_to_apo_default(code):
    """Conversion code jury ScoDoc en code Apogée
    (codes par défaut, c'est configurable via ScoDocSiteConfig.get_code_apo)
    """
    return CODES_SCODOC_TO_APO.get(code, "DEF")


class ScoDocSiteConfig(db.Model):
    """Config. d'un site
    Nouveau en ScoDoc 9: va regrouper les paramètres qui dans les versions
    antérieures étaient dans scodoc_config.py
    """

    __tablename__ = "scodoc_site_config"

    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(128), nullable=False, index=True)
    value = db.Column(db.Text())

    BONUS_SPORT = "bonus_sport_func_name"
    NAMES = {
        BONUS_SPORT: str,
        "always_require_ine": bool,
        "SCOLAR_FONT": str,
        "SCOLAR_FONT_SIZE": str,
        "SCOLAR_FONT_SIZE_FOOT": str,
        "INSTITUTION_NAME": str,
        "INSTITUTION_ADDRESS": str,
        "INSTITUTION_CITY": str,
        "DEFAULT_PDF_FOOTER_TEMPLATE": str,
    }

    def __init__(self, name, value):
        self.name = name
        self.value = value

    def __repr__(self):
        return f"<{self.__class__.__name__}('{self.name}', '{self.value}')>"

    @classmethod
    def get_dict(cls) -> dict:
        "Returns all data as a dict name = value"
        return {
            c.name: cls.NAMES.get(c.name, lambda x: x)(c.value)
            for c in ScoDocSiteConfig.query.all()
        }

    @classmethod
    def set_bonus_sport_func(cls, func_name):
        """Record bonus_sport config.
        If func_name not defined, raise NameError
        """
        if func_name not in cls.get_bonus_sport_func_names():
            raise NameError("invalid function name for bonus_sport")
        c = ScoDocSiteConfig.query.filter_by(name=cls.BONUS_SPORT).first()
        if c:
            log("setting to " + func_name)
            c.value = func_name
        else:
            c = ScoDocSiteConfig(cls.BONUS_SPORT, func_name)
        db.session.add(c)
        db.session.commit()

    @classmethod
    def get_bonus_sport_func_name(cls):
        """Get configured bonus function name, or None if None."""
        f = cls.get_bonus_sport_func_from_name()
        if f is None:
            return ""
        else:
            return f.__name__

    @classmethod
    def get_bonus_sport_func(cls):
        """Get configured bonus function, or None if None."""
        return cls.get_bonus_sport_func_from_name()

    @classmethod
    def get_bonus_sport_func_from_name(cls, func_name=None):
        """returns bonus func with specified name.
        If name not specified, return the configured function.
        None if no bonus function configured.
        Raises ScoValueError if func_name not found in module bonus_sport.
        """
        if func_name is None:
            c = ScoDocSiteConfig.query.filter_by(name=cls.BONUS_SPORT).first()
            if c is None:
                return None
            func_name = c.value
        if func_name == "":  # pas de bonus défini
            return None
        try:
            return getattr(bonus_sport, func_name)
        except AttributeError:
            raise ScoValueError(
                f"""Fonction de calcul maison inexistante: {func_name}. 
                (contacter votre administrateur local)."""
            )

    @classmethod
    def get_bonus_sport_func_names(cls):
        """List available functions names
        (starting with empty string to represent "no bonus function").
        """
        return [""] + sorted(
            [
                getattr(bonus_sport, name).__name__
                for name in dir(bonus_sport)
                if name.startswith("bonus_")
            ]
        )

    @classmethod
    def get_code_apo(cls, code: str) -> str:
        """La représentation d'un code pour les exports Apogée.
        Par exemple, à l'iUT du H., le code ADM est réprésenté par VAL
        Les codes par défaut sont donnés dans sco_apogee_csv.

        """
        cfg = ScoDocSiteConfig.query.filter_by(name=code).first()
        if not cfg:
            code_apo = code_scodoc_to_apo_default(code)
        else:
            code_apo = cfg.value
        return code_apo

    @classmethod
    def set_code_apo(cls, code: str, code_apo: str):
        """Enregistre nouvelle représentation du code"""
        if code_apo != cls.get_code_apo(code):
            cfg = ScoDocSiteConfig.query.filter_by(name=code).first()
            if cfg is None:
                cfg = ScoDocSiteConfig(code, code_apo)
            else:
                cfg.value = code_apo
            db.session.add(cfg)
            db.session.commit()
